/*
    SPDX-FileCopyrightText: 2010 Lamarque Souza <lamarque@kde.org>
    SPDX-FileCopyrightText: 2013 Lukas Tinkl <ltinkl@redhat.com>
    SPDX-FileCopyrightText: 2013-2015 Jan Grulich <jgrulich@redhat.com>

    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
*/

#ifndef MODEMMANAGERQT_GENERIC_TYPES_P_H
#define MODEMMANAGERQT_GENERIC_TYPES_P_H

#include <QMetaType>

Q_DECLARE_METATYPE(ModemManager::MMVariantMapMap)
Q_DECLARE_METATYPE(ModemManager::QVariantMapList)
Q_DECLARE_METATYPE(ModemManager::UIntList)
Q_DECLARE_METATYPE(ModemManager::UIntListList)

#endif // MODEMMANAGERQT_GENERIC_TYPES_P_H
